import {useState} from 'react'
import './App.css'
import {Container, Grid, Typography} from "@mui/material";
import {PDFViewer} from "@react-pdf/renderer";
import {LocalizationProvider} from '@mui/x-date-pickers';
import 'dayjs/locale/de';
import {AdapterDayjs} from '@mui/x-date-pickers/AdapterDayjs'
import MyDocument from "./components/pdf/MyDocument.tsx";
import Person from "./data/Person.ts";
import Bank from "./data/Bank.ts";
import Inputs from "./components/organisms/Inputs.tsx";
import Posten from "./data/Posten.ts";

function App() {
    const [person, setPerson] = useState(new Person());

    const [bank, setBank] = useState(new Bank());

    const postenList = [new Posten(1)];
    const [posten, setPosten] = useState(postenList);

    return (
        <>
            <LocalizationProvider dateAdapter={AdapterDayjs} adapterLocale="de">
                <Container maxWidth={false}>
                    <Grid container>
                        <Grid item lg={12}>
                            <Typography variant={"h1"} align={"center"}>FaSTTUBe Rückerstattung</Typography>
                        </Grid>
                        <Grid item lg={6} sm={12} sx={{height: "90vh", overflow: "auto"}}>
                            <Inputs person={person} setPerson={setPerson} bank={bank} setBank={setBank} posten={posten}
                                    setPosten={setPosten}/>
                        </Grid>
                        <Grid item lg={6} sm={12} sx={{height: "90vh"}}>
                            <PDFViewer style={{height: "100%", width: "100%"}}>
                                <MyDocument person={person} bankdaten={bank} posten={posten}></MyDocument>
                            </PDFViewer>
                        </Grid>
                    </Grid>
                </Container>
            </LocalizationProvider>
        </>
    )
}

export default App
