import {TextField, Select, SelectChangeEvent, MenuItem, Button} from "@mui/material";
import {DatePicker} from '@mui/x-date-pickers/DatePicker';
import DeleteOutlineOutlinedIcon from '@mui/icons-material/DeleteOutlineOutlined';
import Posten from "../../data/Posten.ts";
import Zahlungsart from "../../data/Zahlungsart.ts";
import dayjs from "dayjs";
import {useState} from "react";

const styles = {
    textfield: {
        marginBottom: 1,
        marginRight: 2
    },
    betragfield: {
        marginBottom: 1,
        marginRight: 2,
        maxWidth: 150,
    }
}

const PostenInput = ({thisPosten, posten, setPosten}: { thisPosten: Posten, posten: Posten[], setPosten: any }) => {
    const [betragDisplay, setBetragDisplay] = useState('');

    // useEffect(() => {
    //     // Format the initial posten betrag for display
    //     if (posten.getBetrag()) {
    //         const formattedValue = posten.getBetrag().toString().padStart(3, '0');
    //         const initialDisplayValue = formattedValue.slice(0, -2) + ',' + formattedValue.slice(-2);
    //         setBetragDisplay(initialDisplayValue);
    //     }
    // }, [posten]);

    const handleBetragChange = (event: React.ChangeEvent<HTMLInputElement>, isBlurred: boolean) => {
        const originalValue = event.target.value;
        let value = event.target.value;

        // also work with dots: replace them with commas
        value = value.replace('.', ',');

        // check if comma is in string, if not, use the value as euros (multiply by 100 to get cents)
        if (!value.includes(',')) {
            value = (parseInt(event.target.value) * 100).toString();
        } else {
            // Get characters before and after the comma
            const [euros, cents] = value.split(',');
            // take only first two digits of cents
            const formattedCents = cents.slice(0, 2);
            // if cents are less than 10, pad with 0
            value = euros + formattedCents.padEnd(2, '0');
            // value = (parseInt(euros) * 100 + parseInt(formattedCents)).toString();
        }

        // Allow only digits
        if (/^\d*$/.test(value)) {
            const formattedValue = value.padStart(3, '0'); // Ensure at least 3 digits for formatting
            const displayValue = formattedValue.slice(0, -2) + ',' + formattedValue.slice(-2);

            const newList = posten.map((p) => {
                if (p.getNummer() === thisPosten.getNummer()) {
                    thisPosten.setBetrag(parseInt(formattedValue, 10)); // Update posten in cents
                    return thisPosten;
                }
                return p;
            });
            setPosten(newList);

            if (isBlurred) {
                setBetragDisplay(displayValue);
            } else {
                setBetragDisplay(originalValue);
            }
        }
    };

    // onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
    //     const newPosten = Object.assign(new Posten(1), posten);
    //     newPosten.setBetrag(event.target.value);
    //     setPosten(newPosten);
    // }}/>

    return (
        <>
            <DatePicker label="Datum"
                        sx={styles.textfield}
                        value={thisPosten.getBelegDay()}
                        onChange={(newvalue) => {
                            const newList = posten.map((p) => {
                                if (p.getNummer() === thisPosten.getNummer()) {
                                    thisPosten.setBelegdatum(dayjs(newvalue));
                                    return thisPosten;
                                }
                                return p;
                            });
                            setPosten(newList);
                        }}/>
            <TextField id="outlined-basic" label="Firma" variant="outlined"
                       sx={styles.textfield}
                       value={thisPosten.getFirma()}
                       onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
                           const newList = posten.map((p) => {
                               if (p.getNummer() === thisPosten.getNummer()) {
                                   thisPosten.setFirma(event.target.value);
                                   return thisPosten;
                               }
                               return p;
                           });
                           setPosten(newList);
                       }}/>
            <Select id={"outlined-basic"} label={"Zahlungsart"} variant={"outlined"}
                    sx={styles.textfield}
                    value={thisPosten.getZahlungsart() as unknown as HTMLInputElement}
                    onChange={(event: SelectChangeEvent<HTMLInputElement>) => {
                        const newList = posten.map((p) => {
                            if (p.getNummer() === thisPosten.getNummer()) {
                                thisPosten.setZahlungsart(event.target.value as string);
                                return thisPosten;
                            }
                            return p;
                        });
                        setPosten(newList);
                    }}>
                {
                    Zahlungsart.map((zahlungsart) => {
                        return <MenuItem value={zahlungsart}>{zahlungsart}</MenuItem>
                    })
                }
            </Select>
            <TextField id="outlined-basic" label="Betrag" variant="outlined"
                       sx={styles.betragfield}
                       onBlur={(event: React.FocusEvent<HTMLInputElement>) => {
                           handleBetragChange(event, true);
                       }}
                       onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
                           handleBetragChange(event, false);
                       }}
                       value={betragDisplay}
            />
            <Button variant={"contained"}
                    color={"error"}
                    onClick={() => {
                        let toDeleteFound = false;
                        const newList = posten.map((p) => {
                            if (p.getNummer() === thisPosten.getNummer()) {
                                toDeleteFound = true;
                                return null;
                            }
                            if (!toDeleteFound) {
                                return p;
                            } else {
                                const newNummer = p.getNummer() - 1;
                                p.setNummer(newNummer);
                                return p;
                            }
                        }).filter((p) => p !== null);
                        setPosten(newList);
                    }}
            >
                <DeleteOutlineOutlinedIcon/>
            </Button>
            <TextField id="outlined-basic" label="Begründung" variant="outlined" fullWidth
                       sx={styles.textfield}
                       value={thisPosten.getBegruendung()}
                       onChange={(event: React.ChangeEvent<HTMLInputElement>) => {
                           const newList = posten.map((p) => {
                               if (p.getNummer() === thisPosten.getNummer()) {
                                   thisPosten.setBegruendung(event.target.value);
                                   return thisPosten;
                               }
                               return p;
                           });
                           setPosten(newList);
                       }}/>
        </>
    );
}

export default PostenInput;