import {Page, StyleSheet, Text, View} from "@react-pdf/renderer";
import WarningBox from "./WarningBox.tsx";
import Person from "../../data/Person.ts";
import Bank from "../../data/Bank.ts";
import Posten from "../../data/Posten.ts";

const styles = StyleSheet.create({
    page: {
        flexDirection: 'column',
        backgroundColor: '#ffffff',
        paddingLeft: '2cm',
        paddingRight: '2cm',
        paddingTop: '1.5cm',
        paddingBottom: '1.5cm',
        fontFamily: 'Helvetica',
        fontSize: 11,
    },
    section: {
        margin: 10,
        padding: 10,
        flexGrow: 1,
    },
    header: {
        marginBottom: 20,
    },
    date: {
        textAlign: 'right',
    },
    title: {
        fontSize: 14,
        textAlign: 'center',
        marginBottom: 20,
        fontWeight: 'heavy',
    },
    textBlock: {
        // display: 'flex',
        // flexDirection: 'row',
        // flex: 1,
        // flexWrap: 'wrap',
        marginBottom: 10,
    },
    table: {
        // display: "table",
        width: "auto",
        borderStyle: "solid",
        borderWidth: 1,
        borderRightWidth: 0,
        borderBottomWidth: 0,
        marginTop: 20,
        marginBottom: 10,
    },
    tableRow: {
        margin: "auto",
        flexDirection: "row"
    },
    tableCol: {
        width: "19%",
        borderStyle: "solid",
        borderWidth: 1,
        borderLeftWidth: 0,
        borderTopWidth: 0
    },
    tableColNum: {
        width: "5%",
        borderStyle: "solid",
        borderWidth: 1,
        borderLeftWidth: 0,
        borderTopWidth: 0
    },
    tableColSmall: {
        width: "14%",
        borderStyle: "solid",
        borderWidth: 1,
        borderLeftWidth: 0,
        borderTopWidth: 0
    },
    tableColBig: {
        width: "29%",
        borderStyle: "solid",
        borderWidth: 1,
        borderLeftWidth: 0,
        borderTopWidth: 0
    },
    tableColSum: {
        width: "86%",
        borderStyle: "solid",
        borderWidth: 1,
        borderLeftWidth: 0,
        borderTopWidth: 0
    },
    tableCell: {
        margin: "auto",
        marginTop: 5,
        marginBottom: 5,
        fontSize: 10
    },
    tableCellBetrag: {
        // margin: "auto",
        marginRight: 5,
        marginTop: 5,
        marginBottom: 5,
        fontSize: 10,
        textAlign: 'right',
    },
    signature: {
        marginTop: 60,
        paddingTop: 10,
        borderTopWidth: 1,
        borderTopColor: '#000',
        borderTopStyle: 'solid',
        width: '45%',
        textAlign: 'left',
        fontSize: 9,
    },
    tabbedRow: {
        flexDirection: 'row',
        width: '100%',
    },
    tabbedLeft: {
        width: '10%',
    },
    richtigkeit: {
        marginTop: 40,
    },
});

const AuszahlungPage = ({person, bank, posten}: { person: Person, bank: Bank, posten: Posten[] }) => {
    const current_date = new Date();

    // <View style={styles.tableRow}>
    //     <View style={styles.tableColNum}>
    //         <Text style={styles.tableCell}>{posten.getNummer()}</Text>
    //     </View>
    //     <View style={styles.tableCol}>
    //         <Text style={styles.tableCell}>{posten.getBelegdatum()}</Text>
    //     </View>
    //     <View style={styles.tableCol}>
    //         <Text style={styles.tableCell}>{posten.getFirma()}</Text>
    //     </View>
    //     <View style={styles.tableColBig}>
    //         <Text style={styles.tableCell}>{posten.getBegruendung()}</Text>
    //     </View>
    //     <View style={styles.tableColSmall}>
    //         <Text style={styles.tableCell}>{posten.getZahlungsart()}</Text>
    //     </View>
    //     <View style={styles.tableColSmall}>
    //         <Text style={styles.tableCell}>{posten.getBetragString()}</Text>
    //     </View>
    // </View>

    const day = () => {
        if (current_date.getDate() < 10) {
            return '0' + current_date.getDate();
        }
        return current_date.getDate();
    }
    const month = () => {
        if (current_date.getMonth() + 1 < 10) {
            return '0' + (current_date.getMonth() + 1);
        }
        return current_date.getMonth() + 1;
    }

    const sum = posten.reduce((acc, posten) => acc + posten.getBetrag(), 0);
    const sumEuros = Math.floor(sum / 100);
    const sumCents = sum % 100;
    const centsString = sumCents.toString().padStart(2, "0");
    const sumString = sumEuros + ',' + centsString + ' €';

    return (
        <Page size="A4" wrap={true} style={styles.page}>
            {(person.isCorrect() && bank.isCorrect()) ? null : <WarningBox incorrect={true}/>}
            {(person.isComplete() && bank.isComplete()) ? null : <WarningBox incorrect={undefined}/>}
            <View style={styles.header}>
                <Text>{person.getName()}</Text>
                <Text>{person.getStreet()}</Text>
                <Text>{person.getPlzOrt()}</Text>
                <Text>Mobil: {person.getMobile()}</Text>
                <Text>E-Mail: {person.getEmail()}</Text>
                <Text>Steuer-ID: {person.getTaxId()}</Text>
                <Text>Geburtsdatum: {person.getBirthDate()}</Text>
            </View>
            <View style={styles.header}>
                <Text>Technische Universität Berlin</Text>
                <Text>Fachgebiet Kraftfahrzeuge / TIB 13</Text>
                <Text>Gustav-Meyer-Allee 25</Text>
                <Text>13355 Berlin</Text>
            </View>
            <Text style={styles.date}>Berlin, {day()}.{month()}.{current_date.getFullYear()}</Text>
            <Text style={styles.title}>Auszahlung</Text>
            <Text>Folgende für das studentische Lehrprojekt FaSTTUBe getätigte Ausgaben stelle ich in Rechnung:</Text>
            <View style={styles.table}>
                <View style={styles.tableRow}>
                    <View style={styles.tableColNum}>
                        <Text style={styles.tableCell}>#</Text>
                    </View>
                    <View style={styles.tableCol}>
                        <Text style={styles.tableCell}>Belegdatum</Text>
                    </View>
                    <View style={styles.tableCol}>
                        <Text style={styles.tableCell}>Firma</Text>
                    </View>
                    <View style={styles.tableColBig}>
                        <Text style={styles.tableCell}>Begründung</Text>
                    </View>
                    <View style={styles.tableColSmall}>
                        <Text style={styles.tableCell}>Zahlungsart</Text>
                    </View>
                    <View style={styles.tableColSmall}>
                        <Text style={styles.tableCell}>Betrag</Text>
                    </View>
                </View>
                {posten.map((posten) => {
                    return (
                        <View style={styles.tableRow}>
                            <View style={styles.tableColNum}>
                                <Text style={styles.tableCell}>{posten.getNummer()}</Text>
                            </View>
                            <View style={styles.tableCol}>
                                <Text style={styles.tableCell}>{posten.getBelegdatum()}</Text>
                            </View>
                            <View style={styles.tableCol}>
                                <Text style={styles.tableCell}>{posten.getFirma()}</Text>
                            </View>
                            <View style={styles.tableColBig}>
                                <Text style={styles.tableCell}>{posten.getBegruendung()}</Text>
                            </View>
                            <View style={styles.tableColSmall}>
                                <Text style={styles.tableCell}>{posten.getZahlungsart()}</Text>
                            </View>
                            <View style={styles.tableColSmall}>
                                <Text style={styles.tableCellBetrag}>{posten.getBetragString()}</Text>
                            </View>
                        </View>
                    );
                })}
                <View style={styles.tableRow}>
                    <View style={styles.tableColSum}>
                        <Text style={styles.tableCell}>Summe</Text>
                    </View>
                    <View style={styles.tableColSmall}>
                        <Text style={styles.tableCellBetrag}>{sumString}</Text>
                    </View>
                </View>
            </View>
            <View style={styles.textBlock}>
                <Text style={{marginBottom: 10}}>Bitte überweisen Sie den Betrag auf mein Konto:</Text>
                <View style={styles.tabbedRow}>
                    <Text style={styles.tabbedLeft}>Bank:</Text>
                    <Text>{bank.getName()}</Text>
                </View>
                <View style={styles.tabbedRow}>
                    <Text style={styles.tabbedLeft}>IBAN:</Text>
                    <Text>{bank.getIban()}</Text>
                </View>
                <View style={styles.tabbedRow}>
                    <Text style={styles.tabbedLeft}>BIC:</Text>
                    <Text>{bank.getBic()}</Text>
                </View>
            </View>
            <View style={{flexDirection: 'row', justifyContent: 'space-between'}}>
                <Text style={styles.signature}>Unterschrift Rechnungssteller</Text>
                <Text style={styles.signature}>Unterschrift Finanzleitung</Text>
            </View>
            <Text style={styles.richtigkeit}>Für die Richtigkeit der Leistung / Lieferung:</Text>
            <View style={{flexDirection: 'row', justifyContent: 'space-between'}}>
                <Text style={styles.signature}>Unterschrift, Datum</Text>
                <Text style={styles.signature}>Name in Blockschrift</Text>
            </View>
        </Page>
    );
}

export default AuszahlungPage;
