import IBAN from "iban";

class Bank {
    private name: string;
    private bic: string;
    private iban: string;

    private nameSet: boolean = false;
    private bicSet: boolean = false;
    private ibanSet: boolean = false;

    constructor() {
        this.name = "Deutsche Fakebank AG";
        this.bic = "FAKEDEM1001";
        this.iban = "DE78500105172115399138";
    }

    public getName(): string {
        return this.name;
    }

    public setName(name: string): void {
        this.name = name;

        this.nameSet = name !== "";
    }

    public getBic(): string {
        return this.bic;
    }

    public setBic(bic: string): void {
        this.bic = bic;

        this.bicSet = bic !== "";
    }

    public isBicCorrect(): boolean {
        return this.bic.match(/([a-zA-Z]{4})([a-zA-Z]{2})(([2-9a-zA-Z]{1})([0-9a-np-zA-NP-Z]{1}))((([0-9a-wy-zA-WY-Z]{1})([0-9a-zA-Z]{2}))|([xX]{3})|)/g) !== null;
    }

    public getIban(): string {
        return IBAN.printFormat(this.iban, " ");
    }

    public setIban(iban: string): void {
        this.iban = iban.replace(/ /g, "");

        this.ibanSet = iban !== "";
    }

    public isIbanCorrect(): boolean {
        return IBAN.isValid(this.iban);
    }

    public isComplete(): boolean {
        return this.nameSet && this.bicSet && this.ibanSet;
    }

    public isCorrect(): boolean {
        return this.isBicCorrect() && this.isIbanCorrect();
    }
}

export default Bank;