import dayjs, {Dayjs} from "dayjs";

class Person {
    private name: string;
    private street: string;
    private plzOrt: string;
    private mobile: string;
    private email: string;
    private taxId: string;
    private birthDate: Dayjs;

    private nameSet: boolean = false;
    private streetSet: boolean = false;
    private plzOrtSet: boolean = false;
    private mobileSet: boolean = false;
    private emailSet: boolean = false;
    private taxIdSet: boolean = false;
    private birthDateSet: boolean = false;

    constructor() {
        this.name = "Max Mustermann";
        this.street = "Musterstraße 1";
        this.plzOrt = "12345 Musterstadt";
        this.mobile = "0123 4567890";
        this.email = "max@muster.de";
        this.taxId = "12345678900";
        this.birthDate = dayjs("01.01.1970", "DD.MM.YYYY");
    }

    public getName(): string {
        return this.name;
    }

    public setName(name: string): void {
        this.name = name;

        this.nameSet = name !== "";
    }

    public getStreet(): string {
        return this.street;
    }

    public setStreet(street: string): void {
        this.street = street;

        this.streetSet = street !== "";
    }

    public getPlzOrt(): string {
        return this.plzOrt;
    }

    public setPlzOrt(plzOrt: string): void {
        this.plzOrt = plzOrt;

        this.plzOrtSet = plzOrt !== "";
    }

    public isPlzOrtCorrect(): boolean {
        return this.plzOrt.match(/^\d{5} [a-zA-ZäöüÄÖÜß]+$/g) !== null;
    }

    public getMobile(): string {
        return this.mobile;
    }

    public setMobile(mobile: string): void {
        this.mobile = mobile;

        this.mobileSet = mobile !== "";
    }

    public getEmail(): string {
        return this.email;
    }

    public setEmail(email: string): void {
        this.email = email;

        this.emailSet = email !== "";
    }

    public isEmailCorrect(): boolean {
        return this.email.match(/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/g) !== null;
    }

    public getTaxId(): string {
        // format such that the string is delimited by spaces every 3 digits from the right
        let taxId = this.taxId;
        // reverse the string
        taxId = taxId.split("").reverse().join("");
        // add spaces every 3 digits
        taxId = taxId.replace(/(\d{3})(?=\d)/g, "$1 ");
        // reverse the string back
        taxId = taxId.split("").reverse().join("");
        return taxId;
    }

    public setTaxId(taxId: string): void {
        this.taxId = taxId;

        this.taxIdSet = taxId !== "";
    }

    public isTaxIdCorrect(): boolean {
        return this.taxId.match(/^\d{11}$/g) !== null;
    }

    public getBirthDate(): string {
        return this.birthDate.format("DD.MM.YYYY");
    }

    public setBirthDate(birthDate: Dayjs): void {
        this.birthDate = birthDate;

        this.birthDateSet = birthDate !== dayjs("01.01.1970", "DD.MM.YYYY");
    }

    public isBirthDateCorrect(): boolean {
        // return this.birthDate.match(/^\d{2}\.\d{2}\.\d{4}$/g) !== null;
        // return true;
        return this.birthDate.isValid();
    }

    public isComplete(): boolean {
        return this.nameSet && this.streetSet && this.plzOrtSet && this.mobileSet && this.emailSet && this.taxIdSet && this.birthDateSet;
    }

    public isCorrect(): boolean {
        return this.isPlzOrtCorrect() && this.isEmailCorrect() && this.isTaxIdCorrect() && this.isBirthDateCorrect();
    }
}

export default Person;