import Zahlungsart from "./Zahlungsart.ts";
import dayjs, {Dayjs} from "dayjs";

class Posten {
    private nummer: number;
    private belegdatum: Dayjs;
    private firma: string;
    private begruendung: string;
    private zahlungsart: string;
    private betrag: number;

    constructor(nummer: number) {
        this.nummer = nummer;
        this.belegdatum = dayjs();
        this.firma = "";
        this.begruendung = "";
        this.zahlungsart = Zahlungsart[0];
        this.betrag = 0;
    }

    public assignFrom(posten: Posten): void {
        this.belegdatum = posten.belegdatum;
        this.firma = posten.firma;
        this.begruendung = posten.begruendung;
        this.zahlungsart = posten.zahlungsart;
        this.betrag = posten.betrag;
    }

    public getNummer(): number {
        return this.nummer;
    }

    public setNummer(nummer: number): void {
        this.nummer = nummer;
    }

    public getBelegDay(): Dayjs {
        return this.belegdatum;
    }

    public getBelegdatum(): string {
        return this.belegdatum.format("DD.MM.YYYY");
    }

    public setBelegdatum(belegdatum: Dayjs): void {
        this.belegdatum = belegdatum;
    }

    public getFirma(): string {
        return this.firma;
    }

    public setFirma(firma: string): void {
        this.firma = firma;
    }

    public getBegruendung(): string {
        return this.begruendung;
    }

    public setBegruendung(begruendung: string): void {
        this.begruendung = begruendung;
    }

    public getZahlungsart(): string {
        return this.zahlungsart;
    }

    public setZahlungsart(zahlungsart: string): void {
        this.zahlungsart = zahlungsart;
    }

    public getBetrag(): number {
        return this.betrag;
    }

    public getBetragString(): string {
        // Convert cents to euros without floating point arithmetic
        const euros = Math.floor(this.betrag / 100);
        const cents = this.betrag % 100;
        // left pad cents to 2 digits
        const centsString = cents.toString().padStart(2, "0");
        return euros + "," + centsString + " €";
    }

    public setBetrag(betrag: number): void {
        this.betrag = betrag;
    }
}

export default Posten;
